/**
 * @file
 * PmBridge interface declarations.
 */
 // PmBridge.h

 // C interface to PmBridge.DLL & PmBridge.dylib

#ifndef PMBRIDGE_H
#define PMBRIDGE_H

#ifdef _WIN32

#ifdef PMBRIDGE_EXPORTS
#define DECLSPEC __declspec(dllexport)
#else
#define DECLSPEC __declspec(dllimport)
#endif

#else
#define DECLSPEC
#endif // _WIN32

//
//
//#ifdef _WIN32
//#include "windows.h"
//#endif // _WIN32



// ================================
// Function status return codes
//=================================
#define PMB_OK                       0 ///< No error

// Romexis/Dimaxis has not been successfully started
#define PMB_DX_NOT_RUNNING           1 ///< Romexis/Dimaxis has not been successfully started

// Romexis user/Dimaxis has not been able to login to DB
#define PMB_LOGIN_FAILED             2 ///< Romexis user/Dimaxis has not been able to login to DB

// Specified patient not found in the DB
#define PMB_PATIENT_NOT_FOUND        3 ///< Specified patient not found in the DB

// Failed to insert the patient. Dubl.prim. key?
#define PMB_INSERT_FAILED            4 ///< Failed to insert the patient. Dubl.prim. key?

// Failed to update the patient
#define PMB_UPDATE_FAILED            5 ///< Failed to update the patient

// Failed to delete/in-activate the patient
#define PMB_DELETE_FAILED            6 ///< Failed to delete/in-activate the patient

// (Dimaxis) Can't delete the patient because [s]he has images in the database
#define PMB_DELETE_NOT_ALLOWED       7 ///< (Dimaxis) Can't delete the patient because [s]he has images in the database

// Romexis/Dimaxis has not yet done login to DB (slow LAN may be reason for this)
#define PMB_LOGIN_NOT_YET_OK         8 ///< Romexis/Dimaxis has not yet done login to DB (slow LAN may be reason for this)

// Romexis/Dimaxis is busy (a modal dialog open)
#define PMB_DX_BUSY                  9 ///< Romexis/Dimaxis is busy (a modal dialog open)

// Could not handle image in the given index
#define PMB_INVALID_IMAGE_INDEX     10 ///< Could not handle image in the given index

// No patient has been selected
#define PMB_PATIENT_NOT_SELECTED    11 ///< No patient has been selected

// The selected patient don't have any images
#define PMB_NO_IMAGES_FOUND         12 ///< The selected patient don't have any images

// Failed to get the image from the database
#define PMB_GET_IMAGE_FAILED        13 ///< Failed to get the image from the database

// Failed to write the image to desired location; out of disk space?
#define PMB_WRITE_IMAGE_FAILED      14 ///< Failed to write the image to desired location; out of disk space?

// ID or name missing for DxInsertPatient() or DxUpdatePatient()
#define PMB_REQUIRED_FIELD_MISSING  15 ///< ID or name missing for DxInsertPatient() or DxUpdatePatient()

//  Patient with the same ID and name exists
#define PMB_PATIENT_EXISTS          16 ///< Patient with the same ID and name exists

// Patient with the same ID but different name exists
#define PMB_PATIENT_ID_CONFLICT     17 ///< Patient with the same ID but different name exists

// Image is in long term archive and is not available
#define PMB_IMAGE_IN_LONG_TERM_ARCH 18 ///< Image is in long term archive and is not available

// More than one patient found with the specified patient ID
#define PMB_AMBIGUOUS_PATIENT_ID    19 ///< More than one patient found with the specified patient ID

// If BirthDate is present, format must be YYYYMMDD
#define PMB_INVALID_BIRTH_DATE      20 ///< If BirthDate is present, format must be YYYYMMDD

#define PMB_OPEN_IMAGE_FAILED  		21 
#define PMB_CAPTURE_IMAGE_FAILED  	22 
#define PMB_CAMCAM_CASE_NOT_FOUND   23

#define PMB_CLINIC_STATUS_NOT_FOUND 24
#define PMB_SELECT_MODULE_FAILED    25
#define PMB_GET_IMAGE_DIANOSE_FAILED    26
#define PMB_GET_IMAGE_DIANOSE_NOTFOUND  27
#define PMB_SET_IMAGE_DIANOSE_FAILED    28

#define PMB_INCORRECT_DYNAMIC_PORT  29
#define PMB_UNSPECIFIED_ERROR       30    ///< Used if an unexpected error state is encountered
#define PMB_GET_STUDY_FAILED        31    ///< Failed to get study

#define PMB_GET_PATIENT_INFO_FAILED  32

// =============================================================================
// Modality types (Values on Image parameters array of index DX_INDEX_IMAGE_TYPE)
//==============================================================================
#define DX_IT_PAN        0
#define DX_IT_CEPH       1
#define DX_IT_INTRA      2
#define DX_IT_INTRACAM   3
#define DX_IT_TWAIN      4
#define DX_IT_CR_SCANNER 5
//2D CBCT Snapshot
#define DX_IT_CR            6   ///< 2D CBCT Snapshot
// 3D CBCT Volume
#define DX_IT_CBCT          7  ///< 3D CBCT Volume

#define DX_IT_SURFACE_SCAN  8 ///< Prefer DX_IT_SURFACE_MODEL
#define DX_IT_SURFACE_MODEL 8 ///< Check surface model type from image subtype. Ref. constants with RMX_SMT prefix below
//
#define DX_IT_BINARY        9  ///< Attachment file

#define DX_IT_SMARTPAN      10
#define DX_IT_PLANSCAN      11 ///< Deprecated
#define DX_IT_ORTHO_CASE    12


//=============================================================================
// Surface model image subtypes
//=============================================================================
#define RMX_SMT_UPPER_JAW                1
#define RMX_SMT_TYPE_LOWER_JAW           2
#define RMX_SMT_TYPE_DENTAL_MODEL        4
#define RMX_SMT_TYPE_SEGMENTED_TOOTH     5
#define RMX_SMT_TYPE_SURGI_GUIDE         6
#define RMX_SMT_TYPE_IMPORTED_CROWN      7
#define RMX_SMT_TYPE_PROFACE             8
#define RMX_SMT_TYPE_RENDERING_SNAPSHOT  9
#define RMX_SMT_TYPE_SURGICAL_SPLINT     10
#define RMX_SMT_TYPE_POSTOP_DENTAL_MODEL 11
#define RMX_SMT_TYPE_BASE_SOLID          12
#define RMX_SMT_TYPE_BASE_HOLLOW         13
#define RMX_SMT_TYPE_PROMODEL_IMPLANT    14


//==============================================================================
// Image formats (Values on Image parameters array of index DX_INDEX_FILE_FORMAT)
//===============================================================================
#define DX_IF_TIFF 0
#define DX_IF_JPEG 1
#define DX_IF_BMP  2
#define DX_IF_PNG  3    
#define DX_IF_DICOM 4

//=======================================================================
// Module types supported for remote activation
//===========================================================
#define CMD_PATIENTS      0
#define CMD_FILES         1
#define CMD_CLINIC        2
#define CMD_RECORD        3
#define CMD_XRAY          4
#define CMD_IMG3D         5
#define CMD_EDU           6
#define CMD_CADCAM        7
#define CMD_ADMIN         8
#define CMD_LOGIN         9
#define CMD_SMILEDESIGN  10

//==========================================
// Indices in the image parameter array
// ===========================================
// To be as much as possible language independent, 
// 1 is the first index used
//

//Image Type: 0 ... 9
#define DX_INDEX_IMAGE_TYPE  1  ///<  Image Type: 0 ... 9

// File Format: 0 ... 4
#define DX_INDEX_FILE_FORMAT 2  ///<   File Format: 0 ... 4

#define DX_INDEX_FILE_SIZE   3  ///<

// YYYYMMDD
#define DX_INDEX_IMAGE_DATE  4  ///<  YYYYMMDD

// HHMMSS
#define DX_INDEX_IMAGE_TIME  5  ///<  HHMMSS

// Kilovolts
#define DX_INDEX_VOLTAGE     6  ///<  Kilovolts

// Milliamps
#define DX_INDEX_CURRENT     7  ///<  Milliamps

// Milliseconds
#define DX_INDEX_EXP_TIME    8  ///<  Milliseconds

// Micrometers
#define DX_INDEX_PIXEL_SIZE  9  ///<  Micrometers

// ID for the patient's image in the Romexis/Dimaxis database
#define DX_INDEX_IMAGE_ID   10  ///<  ID for the patient's image in the Romexis/Dimaxis database

#define DX_INDEX_IMAGE_SUBTYPE 11   ///<  subtype of image if exists (0=no subtype) // MKH 2016/05


#define DX_MAX_INDEX 20         ///< Reserve some indices for future use


// MKH 2016/06
// Romexis Image subtypes
#define DX_IST_3D_IMAGE   			1 
//#define DX_IST_2D_SNAPHSOT  		2    ///< 2D view of CBCT Image
//#define DX_IST_IST_FACE_PHOTO  		10   ///< Camera image / photo
#define DX_IST_IST_PAN_BITEWING 	20   ///< Panoramic image taken with bitewing-program

// #define DX_IST_IST_PROSCANNER		50 	 // IntraOral image taken(=imported) with ProScanner
// #define DX_IST_IST_TWAIN			51   // IntraOral image(=imported) taken with TWAIN

#define DX_IST_IST_PROFACE 				100  
#define DX_IST_IST_RENDERING_SNAPSHOT  	110 		// ProFace image created from CBVT rendering
#define DX_IST_IST_IMPORT_MODEL 		111 		// ProFace image created from STL Import
#define DX_IST_IST_IST_MARCHING_CUBES_SURFACE  112  // ProFace image created from STL Import
#define DX_IST_IST_IST_IO_SCAN_IMPLANT 		   113  // ProFace image created from STL Import

#define DX_IST_IST_IST_CEPH_TRACING  	120  		// Ceph analysis or superimposition preview image

#define DX_IST_CADCAM_CASE   				130 // CadCam case file
#define DX_IST_IST_IST_SMILEDESIGN_CASE 	131 // SmileDesign case file
#define DX_IST_IST_IST_SMILEDESIGN_PHOTO 	132 // SmileDesign case file
#define DX_IST_IST_IST_SMILEDESIGN_INTRA  	133 // SmileDesign case file

#define DX_IST_IST_IST_CADCAM_ORDER_XML 	140 // CadCam order file
#define DX_IST_IST_IST_CADCAM_ORDER_PDF  	141 // CadCam case file 



//User's permissions / Access level (NOT used by Romexis)
#define DX_ACL_VIEW_IMAGE  0
// The user is allowed to make exposures and to scan or import images
#define DX_ACL_ACQUIRE_IMAGE  1   
// The user is allowed to delete or move images
#define DX_ACL_DELETE_IMAGE   2
// The user is allowed to modify Dimaxis (ONLY) settings , move images to LTA and edit study templates
#define DX_ACL_ADMINISTRATION 4



//Image Retrieval (= Image Navigator) window size
#define DX_RWND_SIZE_SMALL 0   ///< Image Retrieval (= Image Navigator) window size
#define DX_RWND_SIZE_FULL  1
#define DX_RWND_CLOSE     -1

// Codes for DxShowToolbar / DxShowStatusbar
#define DX_SHOW_BAR  1
#define DX_HIDE_BAR -1

//
// Gender / sex macros
//
// Gender male
#define GENDER_MALE_STRING     "M"   ///< Gender male
// Gender female
#define GENDER_FEMALE_STRING   "F"   ///< Gender female
#define GENDER_UNKNOWN_STRING  "U"   ///< Gender unknown



typedef struct ImageRect {
    int x; // Top left corner
    int y; // Top left corner
    int w; // Width
    int h; // Height
} ImageRect;



// ================
//    FUNCTIONS
// ================
//
#ifdef __cplusplus
extern "C" {
#endif


    // DxStart returns 0 if OK, otherwise error code received from GetLastError()
    DECLSPEC unsigned long  DxStart();

    DECLSPEC short  DxStart2();


    // uShowCmd = { SW_SHOWNORMAL, SW_SHOWMINIMIZED, SW_SHOWMAXIMIZED, ... }
    // DxSetWindowPlacement returns PMB_OK or -1 if function fails
    DECLSPEC short  DxSetWindowPlacement(
        unsigned int uShowCmd);

    // All the following functions return one of the PMB_ error codes


    // Select patient for given patient ID
    // out: number of images
    // NOTE! Returns PMB_BUSY if 2D/3D capture is on!
    DECLSPEC short  DxSelectPatient(
        const char *pszPatientID,  // In
        short *pnImageCount        // Out
    );

    DECLSPEC short  DxSelectPatient_Ext(
        const wchar_t *pwszPatientID,  // In
        short *pnImageCount        // Out
    );

    ///////////////////////////////////////////////////
    /**
     *
     *   INSERT PATIENT- functions
     *
*/
/////////////////////////////////////////////////////
// ANSI
// NOTE! Returns PMB_BUSY if 2D/3D capture is on!
    DECLSPEC short  DxInsertPatient(
        const char *pszPatientID,
        const char *pszFamilyName,
        const char *pszFirstName,
        const char *pszBirthDate,
        const char *pszDoctor
    );

    // ANSI extented function
    // Gender option added
    // NOTE! Returns PMB_BUSY if 2D/3D capture is on!
    DECLSPEC short  DxInsertPatient_Ex(
        const char *pszPatientID,
        const char *pszFamilyName,
        const char *pszFirstName,
        const char *pszBirthDate,
        const char *pszDoctor,
        const char *pszGender
    );

    // WIDE CHAR (=Unicode) function
    // NOTE! Returns PMB_BUSY if 2D/3D capture is on!
    DECLSPEC short  DxInsertPatientW(
        const char  *pszPatientID,
        const wchar_t *pwszFamilyName,
        const wchar_t *pwszFirstName,
        const char  *pszBirthDate,
        const wchar_t *pwszDoctor
    );

    // WIDE CHAR (=Unicode) extended function
    // Gender option added
    // NOTE! Returns PMB_BUSY if 2D/3D capture is on!
    DECLSPEC short  DxInsertPatientW_Ex(
        const char  *pszPatientID,
        const wchar_t *pwszFamilyName,
        const wchar_t *pwszFirstName,
        const char  *pszBirthDate,
        const wchar_t *pwszDoctor,
        const char *pszGender
    );


    //////////////////////////////////////////
    //
    //
    //  UPDATE PATIENT- functions
    //
    //
    ///////////////////////////////////////////

    // ANSI function
    DECLSPEC short  DxUpdatePatient(
        const char *pszPatientID,
        const char *pszFamilyName,
        const char *pszFirstName,
        const char *pszBirthDate,
        const char *pszDoctor
    );

    // ANSI extended function
    // Gender option added
    DECLSPEC short  DxUpdatePatient_Ex(
        const char *pszPatientID,
        const char *pszFamilyName,
        const char *pszFirstName,
        const char *pszBirthDate,
        const char *pszDoctor,
        const char *pszGender
    );

    // WIDE CHAR (=Unicode)  function
    DECLSPEC short  DxUpdatePatientW(
        const char  *pszPatientID,
        const wchar_t *pwszFamilyName,
        const wchar_t *pwszFirstName,
        const char  *pszBirthDate,
        const wchar_t *pwszDoctor
    );

    // WIDE CHAR (=Unicode) extended  function
    // Gender option added
    DECLSPEC short  DxUpdatePatientW_Ex(
        const char  *pszPatientID,
        const wchar_t *pwszFamilyName,
        const wchar_t *pwszFirstName,
        const char  *pszBirthDate,
        const wchar_t *pwszDoctor,
        const char  *pszGender
    );

    ///////////////////////////////////////////////////
    /**
     *
     *   CHANGE PATIENT ID- functions
     *
     * @retval PMB_OK
     * @retval PMB_PATIENT_NOT_FOUND
     * @retval PMB_PATIENT_EXISTS
     * @retval PMB_REQUIRED_FIELD_MISSING
     * @retval PMB_UPDATE_FAILED
     *
     */
     /////////////////////////////////////////////////////
     // ANSI
    DECLSPEC short  DxChangePatientID(
        const char *pszOldPatientID,
        const char *pszNewPatientID
    );

    /////////////////////////////////////////////////////
    // WIDE CHAR (=Unicode) function
    DECLSPEC short  DxChangePatientIDW(
        const wchar_t *pszOldPatientID,
        const wchar_t *pszNewPatientID
    );

    ///////////////////////////////////////////////////
    /**
     *
     *   GET PATIENT INFO- function
     *
     * @retval PMB_OK
     * @retval PMB_PATIENT_NOT_FOUND
     * @retval PMB_GET_PATIENT_INFO_FAILED
     * @retval PMB_REQUIRED_FIELD_MISSING
     * @retval PMB_AMBIGUOUS_PATIENT_ID
     *
     */

     /////////////////////////////////////////////////////
     // WIDE CHAR (=Unicode) function
    DECLSPEC short  DxGetPatientInfoW(
        const wchar_t *pszPatientID, // In
        wchar_t *pszFirstName, // Out. Reserve space for 100 characters
        wchar_t *pszLastName, // Out. Reserve space for 100 characters
        wchar_t *pszBirthDate // Out. Returned in format YYYYMMDD. Reserve space for 9 characters
    );


    /** DxGetPatientIDWCount function
    *
    * @retval PMB_OK
    * @retval PMB_PATIENT_NOT_FOUND
    * @retval PMB_GET_PATIENT_INFO_FAILED
    * @retval PMB_REQUIRED_FIELD_MISSING
    * @retval PMB_AMBIGUOUS_PATIENT_ID
    */
    DECLSPEC short  DxGetPatientIDWCount(
      const wchar_t *firstName, // In
      const wchar_t *lastName,  // In
      const wchar_t *birthdate, // In
      int *nPatients // Out: number of patients matching the search criteria
    );


    /** DxGetPatientID function
     *
     * @retval PMB_OK
     * @retval PMB_PATIENT_NOT_FOUND
     * @retval PMB_GET_PATIENT_INFO_FAILED
     * @retval PMB_REQUIRED_FIELD_MISSING
     * @retval PMB_AMBIGUOUS_PATIENT_ID
     */
    DECLSPEC short  DxGetPatientIDW(
        const wchar_t *firstName, // In
        const wchar_t *lastName,  // In
        const wchar_t *birthdate, // In
        int maxPatients, // In: number of max patient ids to store in the list
        wchar_t **patientIDs  // Out: patient ids
    );

    // *******************************************
    //
    // DELETE PATIENT
    //
    // *******************************************
    DECLSPEC short  DxDeletePatient(
        const char *pszPatientID
    );


    // *******************************************
    //
    //  GET DB STATUS
    //
    // *******************************************
    DECLSPEC short  DxGetDBStatus();


    // *******************************************
    //
    //  EXIT (= close connection)
    //
    // *******************************************
    DECLSPEC short  DxExit();



    /** Explicitly specify the location of the main window of Romexis
     *
     * @param[in] x0      X -coordinate of upper left corner of Romexis window
     * @param[in] y0      Y -coordinate of upper left corner of Romexis window
     * @param[in] nWidth  Width of Romexis main window
     * @param[in] nHeight Height of Romexis main window
     *
     * @retval PMB_OK Operation successful
     * @retval ...
     *
     * @see PmBridge SDK documentation (or PmBridge.h) for possible PMBRIDGE
     * status return values.
     */
    DECLSPEC short  DxMoveWindow(
        long x0,
        long y0,
        long nWidth,
        long nHeight
    );



    // Force Romexis/Dimaxis to be always on top
    //  nOnTop 1: set, 0: reset
    DECLSPEC short  DxAlwaysOnTop(short nOnTop);

    // Start image acquisition of given image type
    DECLSPEC short  DxCaptureImage(short nImageType);

    // Start intra oral image acquisition with or without study
    DECLSPEC short  DxCaptureImageIntra(
        short nUseStudy, short nAllowWithoutStudy
    );

    // Start proscanner image acquisition with or without study
    DECLSPEC short  DxCaptureImageProscan(
        short nUseStudy, short nAllowWithoutStudy
    );

    // Start TWAIN image acquisition with or without study
    DECLSPEC short  DxCaptureImageTWAIN(
        short nUseStudy, short nAllowWithoutStudy
    );


    // Open the Select Image dialog for selecting existing image(s)
    DECLSPEC short  DxSelectImage();


    // Pass worklist information to Romexis/Dimaxis.
    //  - Romexis/Dimaxis selects the patient specified by *pszPatientID
    //  - if *pszModality equals PX, DX or IO Dimaxis opens the corresponding
    //    exposure dialog
    //  - for intraoral exposure, the teeth to be exposured can be passed in
    //    array arrToothMask; indexing in arrToothMask starts from 1 to be
    //    compatible with most development tools
    //
    // Tooth numbers:
    //  Q1 87654321 Q2 12345678
    //  Q4 87654321 Q3 12345678
    //
    // Indices:
    //  8,7,6,5,4,3,2,1            9,10,11,12,13,14,15,16
    //  32,31,30,29,28,27,26,25    17,18,19,20,21,22,23,24
    //
    DECLSPEC short  DxSetWorklistEntry(
        const char *pszPatientID,               // Patient must exist in Romexis/Dimaxis database
        const char *pszStartDate,               // YYYYMMHH
        const char *pszStartTime,               // HHMM<SS>
        const char *pszModality,                // PX = panoramic, IO = intraoral, DX = ceph
        const char *pszStepID,                  // Scheduled Procedure Step ID
        const char *pszStepDesc,                // Scheduled Procedure Step Description
        const char *pszProcedureID,             // Requested Procedure ID
        const char *pszProcedureDesc,   // Requested Procedure Description
        const char *pszStudyInstanceUID,
        const char *pszAccessionNumber,
        short arrToothMask[]                  // Size must be at least 33; 1-based table for dentity
    );


    // Get information about an image
    // DxSelectPatient() returns the number of the images the patient has
    // To be compatible with Visual Basic, the data in the arrays arrImageParam
    //  and arrToothMask starts from index 1
    DECLSPEC short  DxInquireImage(
        short nIndex,            // 0 .. nImageCnt-1
        long  arrImageParam[],  // Size must be at least DX_MAX_INDEX
        short arrToothMask[]   // Size must be at least 33; 1-based table for dentity
    );


    // 2016/05
    // Get info of given CAD/CAM case or all cases for selected patient to file as XML format
    //
    // INPUT:
    //  int nID   				number of case or  0 for all cases
    //  char * pszFileName		full name path of filename (content in XML -format)
    //
    DECLSPEC short  DxInquireCadCamCase(
        const int nID,
        const char *pszFileName
    );

    // Get image as given format using array index
    // (optional enhance image)
    //
    DECLSPEC short  DxGetImage(
        short nIndex,
        short nConvertTo8bit,
        short nEnhance,
        short nFileFormat,
        const char *pszFileName
    );


    // Get image as given format using Romexis image ID
    // (optional enhance image)
    //
    DECLSPEC short  DxGetImageByID(
        int nImageID,
        short nConvertTo8bit,
        short nEnhance,
        short nFileFormat,
        const char *pszFileName
    );


    // Get overview image (thumbnail) as given format using array index
    //
    DECLSPEC short  DxGetOverviewImage(
        short nIndex,
        short nFileFormat,
        const char *pszFileName
    );



    // Get overview image (thumbnail) as given format using Romexis image ID
    //
    DECLSPEC short  DxGetOverviewImageByID(
        int nImageID,
        short nFileFormat,
        const char *pszFileName
    );


    DECLSPEC short  DxSetUser(
        char *pszUserName,          // User name to be recorded with each new image
        unsigned short nAccessLevel  // Access level of the user
    );


    //  Unselects patient
    // NOTE! Returns PMB_BUSY if 2D/3D capture is on!
    DECLSPEC short  DxDeselectPatient();



    // Define limits for the Romexis/Dimaxis window when the window is maximized. Using this function
    //  it is possible to make sure that the calling application's menu is visible when
    //  the window is resized. A value will not be used if it <= 0. The values are relative
    //  to the actual screen size. (This has been tested only in single monitor systems).
    // Dimaxis/Romexis also does a 'sanity' check for the values and uses value only if it does not
    //  limit screen usage more than 10%.
    DECLSPEC short  DxSetMaxLimits(
        long nMinTop,
        long nMaxBottom,
        long nMinLeft,
        long nMaxRight
    );



    // 2016/06/27
    // Planmeca application opens/launches  cadcam case using caseID of the patient opened in Romexis
    //
    // Call first DxSelectPatient. The DxInquireCadCamCase for getting info of caseIDs
    DECLSPEC short  DxOpenCadCamCase(int nCaseID);



    /**
     * Open image in Romexis by giving it's ID or info index
     *
     * NOTE! Following input cases can exists
     * 1. nIndex > -1  and nImageID < 1  => Image opened by image array information
     * 2. nIndex < 0   and nImageID > 0  => Image opened using given ID
     * 3. nIndex > -1  and nImageID > 0  => Image opened using given ID
     * 4. nIndex < 0  and nImageID  < 1  => No image can be opened because of wrong
     *                                      input
     *
     * NOTE2! To be sure that a correct image is opened. One should always call
     *        first DxInquireImage and then DxOpenImage, especially in case 1.
     *
     * @param[in]  nIndex       Index number of image in image info array
     * @param[in]	nImageID    ImageID of image in Romexis database
     *
     * Checking of return value is highly important.
     *
     *
     * @retval PMB_OK Operation successful
     *
     * @see PmBridge SDK documentation (or PmBridge.h) for possible PMBRIDGE status
     *      return values.
     */
    DECLSPEC short  DxOpenImage(short nIndex, int nImageID);



    // 2016/09/17 new function
    //*************************************


    // Planmeca application opens/launches  cadcam case using caseID of the patient opened in Romexis
    //
    // Before calling this function a patient must have been selected (DxSelectPatient)
    DECLSPEC short  DxSelectRmxModule(int nModule);


    //
    // Get clinic status
    //
    DECLSPEC short  DxGetClinicStatus(const char *pszFileName);
    DECLSPEC short  DxGetClinicStatusString(wchar_t *wszStatus);

    //
    // Get image diagnose text of given image
    //
    DECLSPEC short  DxGetImageDiagnose(
        int nImageNumber, wchar_t *pwszText);


    //
    // Set image diagnose text for given image
    //
    DECLSPEC short  DxSetImageDiagnose(
        int nImageNumber, const wchar_t *pwszText);

    DECLSPEC short  DxCloseAllImages();

    // 2018/11/01
    DECLSPEC short  DxGetNumberOfStudies(short *pnStudyCount);

    DECLSPEC short  DxGetStudies(int *studyIDs, int nArrLen);

    DECLSPEC short  DxGetStudies_Name(int arrStudies[], char **arrName,
        int nArrLen);



    /**
     * Get number of images in study. The returned value indicates the max
     * amount of images the given study can contain. To get further information
     * about the actual images, use DxGetStudyInfo method.
     */
    DECLSPEC short  DxGetNumberOfImagesInStudy(
        long studyID, short &nImages);



    /**
     * Fetch study image info to arrays given as parameters.
     *
     * @param[in] studyID
     * @param[out] imageIDs ImageID's of images within the study are stored
     *             here. Array should be allocated the same size as there are
     *             images (nImages). Number of images can be determined with
     *             DxGetNumberOfImagesInStudy function. ID has a value of 0 if
     *             no image data is associated with associated ImageRect.
     * @param[out] imageRects Image locations within a study are stored here as
     *             ImageRect data structures. The size of the array of should
     *             be of size nImages. Images are laid out in a 1920x1080 size
     *             canvas.
     *
     * Checking of return value is highly important.
     *
     * @retval PMB_OK Operation successful
     *
     * @see PmBridge SDK documentation (or PmBridge.h) for possible PMBRIDGE
     *      status return values.
     */
    DECLSPEC short  DxGetStudyInfo(
        long studyID,
        long *imageIDs,
        ImageRect *imageRects);



    DECLSPEC short  DxCaptureStudy(int nStudyID);






    /** DEPRECATED functions */

#ifdef _WIN32
    /** DxShowToolbar and DxShowStatusbar are no longer supported in Romexis
     * 6.0.0.
     * Functions for hiding Toolbar and Statusbar of Dimaxis, to get more room
     * for the images. 1 = show, 0 = hide */
    DECLSPEC short  DxShowToolbar(short nShow);
    DECLSPEC short  DxShowStatusbar(short nShow);

    /** DxSetImageRetrievalWindowSize and DxGetImageRetrievalWindowSize are no
    longer supported in Romexis 6.0.0. */
    DECLSPEC short  DxSetImageRetrievalWindowSize(
       short nWindowSize);
    DECLSPEC short  DxGetImageRetrievalWindowSize(
        short *pnWindowSize,
        long  *pnLeft,
        long  *pnRight,
        long  *pnTop,
        long  *pnBottom);

#endif

#ifdef __cplusplus
}
#endif

#endif // PMBRIDGE_H
