
// PmbWrap.cpp

// Wrapper class for Planmec's PmBridge / implementation

#include "stdafx.h"

#include "PmbWrap.h"

CPmbWrap::CPmbWrap ()
{
	m_pDxStart = NULL ;
	m_pDxSelectPatient = NULL ;
	m_pDxInsertPatientW = NULL ;
	m_pDxUpdatePatientW = NULL ;
	m_pDxDeletePatient = NULL ;
	m_pDxGetDBStatus = NULL ;
	m_pDxExit = NULL ;

	m_pDxInquireImage = NULL ;
	m_pDxInquireCadCamCase = NULL ;

	m_pDxGetImage = NULL ;
	m_pDxOpenImage = NULL ;

	m_pDxGetOverviewImage = NULL ;
	m_pDxSelectImage = NULL ;
	m_pDxCaptureImage = NULL ;
	m_pDxCloseAllImages = NULL;
	m_pDxGetNumsOfStudies = NULL;

	m_pDxSetUser = NULL ;
	m_pDxDeselectPatient = NULL ;

	m_pDxSetWindowPlacement = NULL ;
	m_pDxMoveWindow = NULL;
	m_pDxSetImageRetrievalWindowSize = NULL ;
	m_pDxGetImageRetrievalWindowSize = NULL ;

	m_pDxSetMaxLimits = NULL ;

	m_pDxSetWorklistEntry = NULL ;

	TCHAR szCurrDir [_MAX_PATH] ;
	memset ( szCurrDir, 0, sizeof (szCurrDir) );

	CString strPath ;

	GetDimaxisPath ( strPath );

	if ( strPath > "" )
	{
		if ( GetCurrentDirectory ( sizeof (szCurrDir), szCurrDir ) == 0 )
			memset ( szCurrDir, 0, sizeof (szCurrDir) );

		SetCurrentDirectory ( strPath );

		// There seems to be small bug in the Dimaxis installation program:
		//  the directory "Program" is not included in the registry value.
		// The following call shouldn't make harm even when the installation
		//  script has been corrected

		SetCurrentDirectory ( _T("Program" ) );
	}


	m_hPmbDLL = LoadLibrary ( PMB_DLL );

	if ( strPath > "" )
	{
		// Reset the directory
		SetCurrentDirectory (szCurrDir);
	}

	if ( m_hPmbDLL == NULL )
	{
		AfxGetMainWnd()->MessageBox (PMB_DLL, L"Error", MB_ICONHAND );
		return ;
	}


	m_pDxStart = (pDxStart) GetProcAddress ( m_hPmbDLL, DX_START );

	m_pDxSetWindowPlacement = (pDxSetWindowPlacement) GetProcAddress ( m_hPmbDLL,
		DX_SETWINDOWPLACEMENT );

	m_pDxSelectPatient = (pDxSelectPatient) GetProcAddress ( m_hPmbDLL, DX_SELECT_PATIENT );

	m_pDxInsertPatientW = (pDxInsertPatientW) GetProcAddress ( m_hPmbDLL, DX_INSERT_PATIENTW );
	m_pDxUpdatePatientW = (pDxUpdatePatientW) GetProcAddress ( m_hPmbDLL, DX_UPDATE_PATIENTW );

	m_pDxInsertPatientW_Ex = (pDxInsertPatientW_Ex) GetProcAddress ( m_hPmbDLL, DX_INSERT_PATIENTW_EX );
	m_pDxUpdatePatientW_Ex = (pDxUpdatePatientW_Ex) GetProcAddress ( m_hPmbDLL, DX_UPDATE_PATIENTW_EX );

	m_pDxChangePatientIDW = (pDxChangePatientIDW)GetProcAddress(m_hPmbDLL, DX_CHANGE_PATIENT_IDW);

	m_pDxDeletePatient = (pDxDeletePatient) GetProcAddress ( m_hPmbDLL, DX_DELETE_PATIENT );

	m_pDxGetDBStatus = (pDxGetDBStatus) GetProcAddress ( m_hPmbDLL, DX_GET_DBSTATUS );

	m_pDxExit = (pDxExit) GetProcAddress ( m_hPmbDLL, DX_EXIT );


	m_pDxInquireImage = (pDxInquireImage) GetProcAddress ( m_hPmbDLL, DX_INQUIRE_IMAGE );

	m_pDxGetImage = (pDxGetImage) GetProcAddress ( m_hPmbDLL, DX_GET_IMAGE );
	m_pDxCloseAllImages = (pDxCloseAllImages)GetProcAddress(m_hPmbDLL, DX_CLOSE_ALLIMAGES); // MKH 2019/10/17
	m_pDxGetNumsOfStudies = (pDxGetNumsOfStudies)GetProcAddress(m_hPmbDLL, DX_GET_NUMBEROFSTUDIES); // MKH 2018/11/01
	m_pDxGetStudies = (pDxGetStudies)GetProcAddress(m_hPmbDLL, DX_GET_STUDIES); // MKH 2018/11/01
	m_pDxGetStudies_Name = (pDxGetStudies_Name)GetProcAddress(m_hPmbDLL, DX_GET_STUDIES_NAME); // MKH 2018/11/08
	m_pDxStartStudyCapture = (pDxStartStudyCapture)GetProcAddress(m_hPmbDLL, DX_START_STUDY_CAPTURE); // MKH 2018/11/07

	m_pDxInquireCadCamCase = (pDxInquireCadCamCase) GetProcAddress ( m_hPmbDLL, DX_INQUIRE_CADCAM_CASE );


	m_pDxGetImageByID = (pDxGetImageByID) GetProcAddress ( m_hPmbDLL, DX_GET_IMAGE_BYID ); // MKH 2010/08/05
	
	m_pDxOpenImage = (pDxOpenImage) GetProcAddress ( m_hPmbDLL, DX_OPEN_IMAGE ); // MKH 2010/06/04
	m_pDxOpenCadCamCase = (pDxOpenCadCamCase) GetProcAddress ( m_hPmbDLL, DX_OPEN_CADCAM_CASE ); // 2016/06/23
	m_pDxSelectRmxModule = (pDxSelectRmxModule) GetProcAddress ( m_hPmbDLL, DX_SELECT_MODULE ); // 2016/09

	m_pDxGetClinicStatus = (pDxGetClinicStatus) GetProcAddress ( m_hPmbDLL, DX_GET_CLINIC_STATUS ); // 2016/09
	m_pDxGetClinicStatusStr = (pDxGetClinicStatusStr) GetProcAddress ( m_hPmbDLL, DX_GET_CLINIC_STATUS_STRING ); // 2016/10

	m_pDxGetImageDiagnose = (pDxGetImageDiagnose) GetProcAddress ( m_hPmbDLL, DX_GET_IMAGE_DIAGNOSE ); // 2016/09
	m_pDxSetImageDiagnose = (pDxSetImageDiagnose) GetProcAddress ( m_hPmbDLL, DX_SET_IMAGE_DIAGNOSE ); // 2016/09

	m_pDxGetOverviewImage = (pDxGetOverviewImage) GetProcAddress ( m_hPmbDLL, 
		DX_GET_OVERVIEW_IMAGE );

	// MKH 2010/08/05
	m_pDxGetOverviewImageByID = (pDxGetOverviewImageByID) GetProcAddress ( m_hPmbDLL, 
		DX_GET_OVERVIEW_IMAGE_BYID );

	m_pDxSelectImage = (pDxSelectImage) GetProcAddress ( m_hPmbDLL, DX_SELECT_IMAGE );
	m_pDxCaptureImage = (pDxCaptureImage) GetProcAddress ( m_hPmbDLL, DX_CAPTURE_IMAGE );
	m_pDxCaptureImageIntra = (pDxCaptureImageIntra) GetProcAddress ( m_hPmbDLL, DX_CAPTURE_IMAGE_INTRA );

	m_pDxMoveWindow = (pDxMoveWindow) GetProcAddress ( m_hPmbDLL, DX_MOVE_WINDOW );

	m_pDxSetUser = (pDxSetUser) GetProcAddress ( m_hPmbDLL, DX_SET_USER );

	m_pDxDeselectPatient = (pDxDeselectPatient) GetProcAddress ( m_hPmbDLL, DX_DESELECT_PATIENT );

	m_pDxSetImageRetrievalWindowSize = (pDxSetImageRetrievalWindowSize)  GetProcAddress ( m_hPmbDLL,
		DX_SET_RETRIEVAL_SIZE );

	m_pDxGetImageRetrievalWindowSize = (pDxGetImageRetrievalWindowSize)  GetProcAddress ( m_hPmbDLL,
		DX_GET_RETRIEVAL_SIZE );

	m_pDxSetMaxLimits = (pDxSetMaxLimits) GetProcAddress ( m_hPmbDLL,
		DX_SET_MAX_LIMITS );

	m_pDxSetWorklistEntry = (pDxSetWorklistEntry) GetProcAddress ( m_hPmbDLL,
		DX_SET_WORKLIST_ENTRY );

	if ( m_pDxStart == NULL ||
	 m_pDxSetWindowPlacement == NULL ||
	 m_pDxSelectPatient == NULL ||
	 m_pDxInsertPatientW == NULL  ||
	 m_pDxUpdatePatientW == NULL ||
	 m_pDxInsertPatientW_Ex == NULL  ||
	 m_pDxInquireCadCamCase == NULL  ||
	 m_pDxOpenCadCamCase == NULL  ||
	 m_pDxInsertPatientW_Ex == NULL 
	
	 /*
	 m_pDxDeletePatient == NULL ||
	 m_pDxGetDBStatus == NULL ||
	 m_pDxExit == NULL */ )
	{
		AfxGetMainWnd()->MessageBox (L"Failed to register some of the PmBridge functions", 
			L"Error", MB_ICONHAND );

		FreeLibrary ( m_hPmbDLL );
		m_hPmbDLL = NULL ;

	}
}

boolean CPmbWrap:: LoadPMLibrary()  // MKH 2018/06/04 MKH Entteri test
{
	boolean bRet = true;
	m_hPmbDLL = LoadLibrary(PMB_DLL);
	if (m_hPmbDLL == NULL) {
		bRet = false;
	}
	return bRet;
}

boolean CPmbWrap::FreePMLibrary()  // MKH 2018/06/04 MKH Entteri test
{
	boolean bRet = true;
	bRet = (boolean)FreeLibrary(m_hPmbDLL);
	m_hPmbDLL = NULL;

	return bRet;
}

CPmbWrap::~CPmbWrap ()
{
	if ( m_hPmbDLL != NULL )
		FreeLibrary ( m_hPmbDLL );
}



//////////////////////////////////////////////////////////////////////////////

void CPmbWrap::GetDimaxisPath ( CString & strPath )
{
	const WCHAR *pszKeyName = L"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\App Paths\\Dimaxis.exe" ;

	const WCHAR *pszValName = L"Path" ;

	HKEY hKeyRoot = HKEY_LOCAL_MACHINE, hKeyDx ;
	PDWORD pReserved = NULL ;
	const REGSAM samDesired = KEY_READ ;
	const DWORD ulOptions = 0 ;
	DWORD dwValueType = REG_SZ ;
	TCHAR szPath [_MAX_PATH] ;
	DWORD dwDataSize = sizeof (szPath) ;

	DWORD dwRet = RegOpenKeyEx ( hKeyRoot, pszKeyName, ulOptions, samDesired,
		&hKeyDx );		

	if ( dwRet != NO_ERROR )
	{
		AfxGetMainWnd()->MessageBox (L"Dimaxis key/path not found in the registry",
			NULL, MB_ICONEXCLAMATION );
		strPath = "" ;
		return ;
	}

	dwRet = RegQueryValueEx ( hKeyDx, pszValName, pReserved, &dwValueType,
		(PBYTE) szPath, &dwDataSize );

	if ( dwRet != NO_ERROR )
	{
		AfxGetMainWnd()->MessageBox ( L"Dimaxis path not found in the registry",
			NULL, MB_ICONEXCLAMATION );
		strPath = "" ;
		RegCloseKey ( hKeyDx );
		return ;
	}
	strPath.GetBuffer (_MAX_PATH);

	strPath = szPath ;
	RegCloseKey ( hKeyDx );
}



//////////////////////////////////////////////////////////////////////////////


short CPmbWrap::StartDx ()
{
	if ( m_hPmbDLL != NULL && m_pDxStart != NULL )
		return m_pDxStart ();

	return -1 ;
}


short CPmbWrap::SetDxWindowPlacement (
	UINT uShowCmd
	)
{
	if ( m_hPmbDLL != NULL && m_pDxSetWindowPlacement != NULL )
		return m_pDxSetWindowPlacement ( uShowCmd );

	return -1 ;
}

short CPmbWrap::SelectPatient (
	const char *pszPatientID,
	short *pnImageCount
	)
{
	if ( m_hPmbDLL != NULL && m_pDxSelectPatient != NULL )
	{
		return m_pDxSelectPatient ( pszPatientID, pnImageCount );
	}

	return -1 ;
}

short CPmbWrap::InsertPatientW (
	const char  *pszPatientID,
	const WCHAR *pwszFamilyName,
	const WCHAR *pwszFirstName,
	const char  *pszBirthDate,
	const WCHAR *pwszDoctor
	)
{
	if ( m_hPmbDLL != NULL && m_pDxInsertPatientW != NULL )
		return m_pDxInsertPatientW ( pszPatientID, pwszFamilyName, pwszFirstName,
			pszBirthDate, pwszDoctor );

	return -1 ;
}

short CPmbWrap::InsertPatientW_Ex (
	const char  *pszPatientID,
	const WCHAR *pwszFamilyName,
	const WCHAR *pwszFirstName,
	const char  *pszBirthDate,
	const WCHAR *pwszDoctor,
	const char *pszGender
	)
{
	if ( m_hPmbDLL != NULL && m_pDxInsertPatientW_Ex != NULL )
		return m_pDxInsertPatientW_Ex ( pszPatientID, pwszFamilyName, pwszFirstName,
			pszBirthDate, pwszDoctor,pszGender);

	return -1 ;
}


short CPmbWrap::UpdatePatientW (
	const char  *pszPatientID,
	const WCHAR *pwszFamilyName,
	const WCHAR *pwszFirstName,
	const char  *pszBirthDate,
	const WCHAR *pwszDoctor
	)
{
	if ( m_hPmbDLL != NULL && m_pDxUpdatePatientW != NULL )
		return m_pDxUpdatePatientW ( pszPatientID, pwszFamilyName, pwszFirstName,
			pszBirthDate, pwszDoctor );

	return -1 ;
}

short CPmbWrap::UpdatePatientW_Ex (
	const char  *pszPatientID,
	const WCHAR *pwszFamilyName,
	const WCHAR *pwszFirstName,
	const char  *pszBirthDate,
	const WCHAR *pwszDoctor,
	const char *pszGender
	)
{
	if ( m_hPmbDLL != NULL && m_pDxUpdatePatientW_Ex != NULL )
		return m_pDxUpdatePatientW_Ex ( pszPatientID, pwszFamilyName, pwszFirstName,
			pszBirthDate, pwszDoctor,pszGender );

	return -1 ;
}

short CPmbWrap::ChangePatientIDW(
	const wchar_t *oldExternalID, const wchar_t *newExternalID
) {
	if (m_hPmbDLL != NULL && m_pDxChangePatientIDW != NULL) {
		return m_pDxChangePatientIDW(oldExternalID, newExternalID);
	}
	return -1;
}

short CPmbWrap::DeletePatient (
	const char *pszPatientID
	)
{
	if ( m_hPmbDLL != NULL && m_pDxDeletePatient != NULL )
		m_pDxDeletePatient ( pszPatientID );

	return -1 ;
}


short CPmbWrap::GetDBStatus (
	)
{
	if ( m_hPmbDLL != NULL && m_pDxGetDBStatus != NULL )
		return m_pDxGetDBStatus ();

	// Function not yet available in the PmBridge.DLL version in use.
	// Assume that the connection is ok.
	return 0 ;
}


short CPmbWrap::ExitDx (
	)
{
	if ( m_hPmbDLL != NULL && m_pDxExit != NULL )
		return m_pDxExit ();

	return -1 ;
}



short CPmbWrap::InquireImage (
	short nIndex,
	long  arrImageParam [],
	short arrToothMask  []
	)
{
	if ( m_hPmbDLL != NULL && m_pDxInquireImage != NULL )
		return m_pDxInquireImage (nIndex, arrImageParam, arrToothMask);

	return -1 ;
}


short CPmbWrap::GetImage (
	short nIndex,
	short nConvertTo8bit,
	short nEnhance,
	short nFileFormat,
	const char *pszFileName
	)
{
	if ( m_hPmbDLL != NULL && m_pDxGetImage != NULL )
		return m_pDxGetImage ( nIndex, nConvertTo8bit, nEnhance, nFileFormat, pszFileName );

	return -1 ;
}


short CPmbWrap::CloseAllImages()
{
	if (m_hPmbDLL != NULL && m_pDxCloseAllImages != NULL)
		return m_pDxCloseAllImages();

	return -1;
}

short CPmbWrap::GetNumbOfStudies(short * nNums)
{
	if (m_hPmbDLL != NULL && m_pDxGetNumsOfStudies != NULL)
		return m_pDxGetNumsOfStudies(nNums);

	return -1;
}

short CPmbWrap::GetStudies(int *arrStudy, int nNums)
{
	if (m_hPmbDLL != NULL && m_pDxGetStudies != NULL)
		return m_pDxGetStudies(arrStudy,nNums);

	return -1;
}

short CPmbWrap::GetStudiesWithName(int *arrStudy,char **arrNames , int nNums)
{
	if (m_hPmbDLL != NULL && m_pDxGetStudies_Name != NULL)
		return m_pDxGetStudies_Name(arrStudy, arrNames, nNums);

	return -1;
}

short CPmbWrap::StartStudyCapture(int nStudyID)
{
	if (m_hPmbDLL != NULL && m_pDxStartStudyCapture != NULL)
		return m_pDxStartStudyCapture(nStudyID);

	return -1;

}

short CPmbWrap::InquireCadCamCase (
	const int nID,
	const char *pszFileName
	)
{
	if ( m_hPmbDLL != NULL && m_pDxInquireCadCamCase != NULL )
		return m_pDxInquireCadCamCase ( nID,pszFileName);

	return -1 ;
}

// MKH 2010/08/05
short CPmbWrap::GetImageByID (
	int nImageID,
	short nConvertTo8bit,
	short nEnhance,
	short nFileFormat,
	const char *pszFileName
	)
{
	if ( m_hPmbDLL != NULL && m_pDxGetImageByID != NULL )
		return m_pDxGetImageByID ( nImageID, nConvertTo8bit, nEnhance, nFileFormat, pszFileName );

	return -1 ;
}


short CPmbWrap::SelectRmxModule(int nModule) 
{
	if ( m_hPmbDLL != NULL && m_pDxSelectRmxModule != NULL )
		return m_pDxSelectRmxModule(nModule);

	return -1 ;
}

short CPmbWrap::GetClinicStatus(const char * szFileName) 
{
	if ( m_hPmbDLL != NULL && m_pDxGetClinicStatus != NULL )
		return m_pDxGetClinicStatus(szFileName);
	return -1 ;
}

short CPmbWrap::GetClinicStatusStr(WCHAR * pwszStatus) 
{
	if ( m_hPmbDLL != NULL && m_pDxGetClinicStatusStr != NULL )
		return m_pDxGetClinicStatusStr(pwszStatus);
	return -1 ;
}


short CPmbWrap::GetImageDiagnose(int nImageNumber,  WCHAR * pwszDiagnose) 
{
	if ( m_hPmbDLL != NULL && m_pDxGetImageDiagnose != NULL )
		return m_pDxGetImageDiagnose(nImageNumber,pwszDiagnose);

	return -1 ;
}

short CPmbWrap::SetImageDiagnose(int nImageNumber, const WCHAR * pwszText) 
{
	if ( m_hPmbDLL != NULL && m_pDxSetImageDiagnose != NULL )
		return m_pDxSetImageDiagnose(nImageNumber,pwszText);

	return -1 ;
}


short CPmbWrap::OpenImage (
	short nIndex,
	int nID
	)
{
	if ( m_hPmbDLL != NULL && m_pDxOpenImage != NULL )
		return m_pDxOpenImage ( nIndex,nID );

	return -1 ;
}


short CPmbWrap::OpenCadCamCase (
	int nCaseID
	)
{
	if ( m_hPmbDLL != NULL && m_pDxOpenCadCamCase != NULL )
		return m_pDxOpenCadCamCase (nCaseID );

	return -1 ;
}

short CPmbWrap::GetOverviewImage (
	short nIndex,
	short nFileFormat,
	const char *pszFileName
	)
{
	if ( m_hPmbDLL != NULL && m_pDxGetOverviewImage != NULL )
		return m_pDxGetOverviewImage ( nIndex, nFileFormat, pszFileName );

	return -1 ;
}


// MKH 2010/08/05
short CPmbWrap::GetOverviewImageByID (
	int nImageID,
	short nFileFormat,
	const char *pszFileName
	)
{
	if ( m_hPmbDLL != NULL && m_pDxGetOverviewImageByID != NULL )
		return m_pDxGetOverviewImageByID ( nImageID, nFileFormat, pszFileName); //,nEnhance );

	return -1 ;
}


short CPmbWrap::SelectImage ()
{
	if ( m_hPmbDLL != NULL && m_pDxSelectImage != NULL )
		return m_pDxSelectImage ();

	return -1 ;
}


short CPmbWrap::CaptureImage (
	short nImageType
	)
{
	if ( m_hPmbDLL != NULL && m_pDxCaptureImage != NULL )
		return m_pDxCaptureImage ( nImageType );

	return -1 ;
}


short CPmbWrap::CaptureImageIntra (
	short nUseStudy, short nAllowWithoutStudy
	)
{
	if ( m_hPmbDLL != NULL && m_pDxCaptureImageIntra != NULL )
		return m_pDxCaptureImageIntra ( nUseStudy, nAllowWithoutStudy );

	return -1 ;
}


short CPmbWrap::SetUser (
	const char *pszUserName,
	unsigned short nAccessLevel
	)
{
	if ( m_hPmbDLL != NULL && m_pDxSetUser != NULL )
		return m_pDxSetUser ( pszUserName, nAccessLevel );

	return -1 ;
}


short CPmbWrap::DeselectPatient ()
{
	if ( m_hPmbDLL != NULL && m_pDxDeselectPatient != NULL )
		return m_pDxDeselectPatient ();

	return -1 ;
}


short CPmbWrap::SetImageRetrievalWindowSize ( short nSize )
{
	if ( m_hPmbDLL != NULL && m_pDxSetImageRetrievalWindowSize != NULL )
		return m_pDxSetImageRetrievalWindowSize ( nSize );

	return -1 ;
}


short CPmbWrap::GetImageRetrievalWindowSize ( 
	short *pnWindowSize,
	long  *pnLeft,
	long  *pnRight,
	long  *pnTop,
	long  *pnBottom
	)
{
	if ( m_hPmbDLL != NULL && m_pDxGetImageRetrievalWindowSize != NULL )
		return m_pDxGetImageRetrievalWindowSize ( pnWindowSize,
			pnLeft, pnRight, pnTop, pnBottom );

	return -1 ;
}



short CPmbWrap::SetMaxLimits (
	long nMinTop,
	long nMaxBottom,
	long nMinLeft,
	long nMaxRight
	)
{
	if ( m_hPmbDLL != NULL && m_pDxSetMaxLimits != NULL )
		return m_pDxSetMaxLimits ( nMinTop,	nMaxBottom, nMinLeft, nMaxRight );

	return -1 ;
}



short CPmbWrap::SetWorklistEntry (
	const char *pszPatientID,
	const char *pszStartDate,
	const char *pszStartTime,
	const char *pszModality,
	const char *pszStepID,
	const char *pszStepDesc,
	const char *pszProcedureID,
	const char *pszProcedureDesc,
	const char *pszStudyInstanceUID,
	const char *pszAccessionNumber,
	short arrToothMask  []
	)
{
	if ( m_hPmbDLL != NULL && m_pDxSetWorklistEntry != NULL )
		return m_pDxSetWorklistEntry ( pszPatientID, pszStartDate, pszStartTime,
			pszModality, pszStepID, pszStepDesc, pszProcedureID, pszProcedureDesc,
			pszStudyInstanceUID, pszAccessionNumber, arrToothMask );

	return -1 ;
}


